@@ -2,6 +2,7 @@ import json
 import logging
 import math
 import tempfile
+from dataclasses import field
 from os import path
 from typing import List, Optional
 from zipfile import ZIP_DEFLATED, ZipFile
@@ -11,7 +12,7 @@ from pydantic.dataclasses import dataclass
 
 from .logger import LOG_FILE, get_logger
 from .sink import Sink
-from .storage import BaseStorage, LocalStorage, S3Storage
+from .storage import BaseStorage, LocalStorage, S3Storage, S3StorageConfig
 
 logger = get_logger(__name__)
 
@@ -31,6 +32,9 @@ class FileSinkConfig:
     # IAM role to assume before writing to file
     assume_role_arn: Optional[str] = None
 
+    # IAM credential to access S3 bucket
+    s3_auth_config: S3StorageConfig = field(default_factory=lambda: S3StorageConfig())
+
 
 class FileSink(Sink):
     """File sink functions"""
@@ -41,7 +45,9 @@ class FileSink(Sink):
         self.write_logs = config.write_logs
 
         if config.directory.startswith("s3://"):
-            self._storage: BaseStorage = S3Storage(config.assume_role_arn)
+            self._storage: BaseStorage = S3Storage(
+                config.assume_role_arn, config.s3_auth_config
+            )
         else:
             self._storage = LocalStorage()
 
