@@ -1,17 +1,18 @@
 import logging
 import math
-from datetime import datetime, timedelta, timezone
+from datetime import datetime
 from typing import Collection, Dict, List, Tuple
 
 from metaphor.models.metadata_change_event import DataPlatform, Dataset
 from pydantic import parse_raw_as
-from pydantic.dataclasses import dataclass
 
 from metaphor.common.event_util import ENTITY_TYPES
 from metaphor.common.extractor import BaseExtractor
 from metaphor.common.filter import DatabaseFilter
 from metaphor.common.logger import get_logger
 from metaphor.common.usage_util import UsageUtil
+from metaphor.common.utils import start_of_day
+from metaphor.snowflake.accessed_object import AccessedObject
 from metaphor.snowflake.auth import connect
 from metaphor.snowflake.usage.config import SnowflakeUsageRunConfig
 from metaphor.snowflake.utils import QueryWithParam, async_execute
@@ -22,20 +23,6 @@ logger = get_logger(__name__)
 logging.getLogger("Parser").setLevel(logging.CRITICAL)
 
 
-@dataclass
-class AccessedObjectColumn:
-    columnId: int
-    columnName: str
-
-
-@dataclass
-class AccessedObject:
-    objectDomain: str
-    objectName: str
-    objectId: int
-    columns: List[AccessedObjectColumn]
-
-
 DEFAULT_EXCLUDED_DATABASES: DatabaseFilter = {"SNOWFLAKE": None}
 
 
@@ -51,6 +38,7 @@ class SnowflakeUsageExtractor(BaseExtractor):
         self.filter = None
         self.max_concurrency = None
         self.batch_size = None
+        self._use_history = True
         self._datasets: Dict[str, Dataset] = {}
 
     async def extract(
@@ -63,6 +51,7 @@ class SnowflakeUsageExtractor(BaseExtractor):
         self.account = config.account
         self.max_concurrency = config.max_concurrency
         self.batch_size = config.batch_size
+        self._use_history = config.use_history
 
         self.filter = config.filter.normalize()
 
@@ -78,7 +67,8 @@ class SnowflakeUsageExtractor(BaseExtractor):
             else ""
         )
 
-        start_date = datetime.utcnow().date() - timedelta(config.lookback_days)
+        lookback_days = 1 if config.use_history else config.lookback_days
+        start_date = start_of_day(lookback_days)
 
         conn = connect(config)
 
@@ -129,8 +119,9 @@ class SnowflakeUsageExtractor(BaseExtractor):
                 self._parse_access_logs,
             )
 
-        # calculate statistics based on the counts
-        UsageUtil.calculate_statistics(self._datasets.values())
+        if not self._use_history:
+            # calculate statistics based on the counts
+            UsageUtil.calculate_statistics(self._datasets.values())
 
         return self._datasets.values()
 
@@ -143,6 +134,13 @@ class SnowflakeUsageExtractor(BaseExtractor):
         try:
             objects = parse_raw_as(List[AccessedObject], accessed_objects)
             for obj in objects:
+                if not obj.objectDomain or obj.objectDomain.upper() not in (
+                    "TABLE",
+                    "VIEW",
+                    "MATERIALIZED VIEW",
+                ):
+                    continue
+
                 table_name = obj.objectName.lower()
                 parts = table_name.split(".")
                 db, schema, table = parts[0], parts[1], parts[2]
@@ -150,17 +148,27 @@ class SnowflakeUsageExtractor(BaseExtractor):
                     logger.debug(f"Ignore {table_name} due to filter config")
                     continue
 
+                utc_now = start_of_day()
+
                 if table_name not in self._datasets:
                     self._datasets[table_name] = UsageUtil.init_dataset(
-                        self.account, table_name, DataPlatform.SNOWFLAKE
+                        self.account,
+                        table_name,
+                        DataPlatform.SNOWFLAKE,
+                        self._use_history,
+                        utc_now,
                     )
 
                 columns = [column.columnName.lower() for column in obj.columns]
 
-                utc_now = datetime.now().replace(tzinfo=timezone.utc)
-                UsageUtil.update_table_and_columns_usage(
-                    self._datasets[table_name].usage, columns, start_time, utc_now
-                )
+                if self._use_history:
+                    UsageUtil.update_table_and_columns_usage_history(
+                        self._datasets[table_name].usage_history, columns
+                    )
+                else:
+                    UsageUtil.update_table_and_columns_usage(
+                        self._datasets[table_name].usage, columns, start_time, utc_now
+                    )
         except Exception as e:
             logger.error(f"access log error, objects: {accessed_objects}")
             logger.exception(e)
